/*
 * osdlistitem.cpp
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#include "osdlistitem.h"



OsdListItem::OsdListItem( OsdConfig *conf, const QString &text ) : OsdObject( conf )
{
	label = text;
	labelTexture = 0;
}



OsdListItem::~OsdListItem()
{
	if ( labelTexture )
		glDeleteTextures( 1, &labelTexture );
}



void OsdListItem::mouseMove( QPoint &p )
{
	bool oldState = hasFocus;

	if ( inMe( p.x(), p.y() ) )
		hasFocus = true;
	else
		hasFocus = false;

	if ( oldState!=hasFocus )
		emit wantRepaint();
}



bool OsdListItem::mouseClicked( QMouseEvent *e )
{
	if ( !inMe( e->x(), e->y() ) )
		return false;
	emit clicked( label );
	return true;
}



void OsdListItem::paint()
{
	if ( !labelTexture ) {
		glGenTextures( 1, &labelTexture );
		QImage img( rect.width(), rect.height()*2, QImage::Format_ARGB32 );
		QPainter p;
		p.begin( &img );
		memset( img.bits(), 0, img.width()*img.height()*4 );
		p.setFont( config->normalFont() );

		QLinearGradient grad( QPoint(0,0), QPoint(img.width(),0) );
		grad.setColorAt(1, config->listItemGrad1Color() );
		grad.setColorAt(0, config->listItemGrad0Color() );
		p.fillRect( 0, 0, img.width(), rect.height(), grad );
		p.setPen( config->foregroundColor() );
		p.drawText( TEXTMARGIN, 0, rect.width()-(2*TEXTMARGIN), rect.height() , Qt::AlignVCenter, label );

		grad = QLinearGradient( QPoint(0,rect.height()), QPoint(img.width(),img.height()) );
		grad.setColorAt(1, config->listItemHighGrad1Color() );
		grad.setColorAt(0, config->listItemHighGrad0Color() );
		p.fillRect( 0, rect.height(), img.width(), rect.height(), grad );
		p.setPen( config->foregroundHighColor() );
		p.drawText( TEXTMARGIN, rect.height(), rect.width()-(2*TEXTMARGIN), rect.height() , Qt::AlignVCenter, label );

		p.end();

		int texW, texH;
		config->func->textureFromData( labelTexture, img.width(), img.height(), img.bits(), texW, texH );
		lu = 0;
		lu1 = (float)img.width()/(float)texW;
		lv = 0;
		lv1 = (float)rect.height()/(float)texH;
		lv2 = (float)img.height()/(float)texH;
	}
	GLfloat nv, nv1;
	if ( hasFocus ) {
		nv = lv1;
		nv1 = lv2;
	}
	else {
		nv = lv;
		nv1 = lv1;
	}
	glColor4f( 1, 1, 1, 1 );
	config->func->drawQuadTexture( labelTexture, rect, lu, nv, lu1, nv1 );
}
