/*
 * shaders.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef SHADERS_H
#define SHADERS_H

	// yuv 4:2:0
	const char *FPyv12=
	"!!ARBfp1.0\n"
	"TEMP tmp, yuv;\n"
	"TEX yuv.r, fragment.texcoord[0], texture[0], 2D;\n"
	"TEX yuv.g, fragment.texcoord[0], texture[1], 2D;\n"
	"TEX yuv.b, fragment.texcoord[0], texture[2], 2D;\n"
	"SUB yuv, yuv, { 0.0625, 0.5, 0.5 };\n"
	"MUL yuv.r, yuv.r, 1.1643;\n"
	"MAD result.color.r, 1.5958, yuv.b, yuv.r;\n"
	"MUL tmp, yuv, { 0, 0.39173, 0.8129 };\n"
	"ADD tmp.r, tmp.g, tmp.b;\n"
	"SUB result.color.g, yuv.r, tmp.r;\n"
	"MAD result.color.b, 2.017, yuv.g, yuv.r;\n"
	"MOV result.color.a, 1.0;\n"
	"END";

	// yuv 4:2:2
	const char *FPyuy2=
	"!!ARBfp1.0\n"
	"TEMP tmp, yuv;\n"
	"TEX yuv.r, fragment.texcoord[0], texture[0], 2D;\n"
	"TEX tmp, fragment.texcoord[0], texture[1], 2D;\n"
	"MOV yuv.g, tmp.g;\n"
	"MOV yuv.b, tmp.a;\n"
	"SUB yuv, yuv, { 0.0625, 0.5, 0.5 };\n"
	"MUL yuv.r, yuv.r, 1.1643;\n"
	"MAD result.color.r, 1.5958, yuv.b, yuv.r;\n"
	"MUL tmp, yuv, { 0, 0.39173, 0.8129 };\n"
	"ADD tmp.r, tmp.g, tmp.b;\n"
	"SUB result.color.g, yuv.r, tmp.r;\n"
	"MAD result.color.b, 2.017, yuv.g, yuv.r;\n"
	"MOV result.color.a, 1.0;\n"
	"END";

	// With this shader, yuv 4:2:2 (YCbY2Cr) data are loaded as luminance_alpha (YCb, Y2Cr ..).
	// So we use rectangle texture to check for even/odd texel.
	// We also have to set filter to GL_NEAREST and do blerp in shader.
	// This requires strong GPU but reduces the amount of texture data to be loaded (x2).
	const char *FSyuy2=
	"uniform sampler2DRect YUVtex;\n"
	"void main(void) {\n"
	"  float bl;\n"
	"  vec2 coord, x1, x2, x3, x4;\n"
	"  vec3 yuv;\n"
	"  vec4 pbl, tmp, v13, v24;\n"
	"  coord = gl_TexCoord[0].xy;\n"
	"  tmp = vec4(floor(coord.x), floor(coord.y), ceil(coord.x), ceil(coord.y));\n"
	"  x1=tmp.xy; x2=tmp.zy; x3=tmp.xw; x4=tmp.zw;\n"
	"  pbl.x = 1.0/distance( x1, coord );\n"
	"  pbl.y = 1.0/distance( x2, coord );\n"
	"  pbl.z = 1.0/distance( x3, coord );\n"
	"  pbl.w = 1.0/distance( x4, coord );\n"
	"  tmp.x = v13.xy = texture2DRect(YUVtex,x1).ra;\n"
	"  tmp.y = v24.xy = texture2DRect(YUVtex,x2).ra;\n"
	"  tmp.z = v13.zw = texture2DRect(YUVtex,x3).ra;\n"
	"  tmp.w = v24.zw = texture2DRect(YUVtex,x4).ra;\n"
	"  bl = pbl.x+pbl.y+pbl.z+pbl.w;\n"
	"  yuv.r = dot(tmp, pbl)/bl;\n"
	"  if ( fract(x1.x/2.0)!=0.0 ) {\n"
	"    tmp.x = texture2DRect(YUVtex,vec2(x1.x-1.0,x1.y)).a;\n"
	"    tmp.y = v24.y;\n"
	"    tmp.z = texture2DRect(YUVtex,vec2(x3.x-1.0,x3.y)).a;\n"
	"    tmp.w = v24.w;\n"
	"    yuv.g = dot(tmp, pbl)/bl;\n"
	"    tmp.x = v13.y;\n"
	"    tmp.y = texture2DRect(YUVtex,vec2(x2.x+1.0,x2.y)).a;\n"
	"    tmp.z = v13.w;\n"
	"    tmp.w = texture2DRect(YUVtex,vec2(x4.x+1.0,x4.y)).a;\n"
	"    yuv.b = dot(tmp, pbl)/bl;\n"
	"  }\n"
	"  else {\n"
	"    tmp.xy = v13.y;\n"
	"    tmp.zw = v13.w;\n"
	"    yuv.g = dot(tmp, pbl)/bl;\n"
	"    tmp.xy = v24.y;\n"
	"    tmp.zw = v24.w;\n"
	"    yuv.b = dot(tmp, pbl)/bl;\n"
	"  }\n"
	"  yuv = yuv-vec3(0.0625, 0.5, 0.5);\n"
  	"  yuv.r = yuv.r*1.1643;\n"
	"  gl_FragColor=vec4(yuv.r+1.5958*yuv.b, yuv.r-0.39173*yuv.g-0.8129*yuv.b, yuv.r+2.017*yuv.g, 1.0);\n"
	"}\n";

#endif // SHADERS_H
