/*
 * videowidget.h
 *
 * Copyright (C) 2008 Christophe Thommeret <hftom@free.fr>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA
 */

#ifndef VIDEOWIDGET_H
#define VIDEOWIDGET_H

#include <QList>
#include <QTimer>

#include "glvideo.h"
#include "xineengine.h"
#include "xinethumb.h"

class Osd;



class VideoWidget : public QGLWidget
{
	Q_OBJECT
public:

	VideoWidget( QWidget *parent, QString mrl );
	~VideoWidget();

	QString shot();

	static void raw_output_cb(void *user_data, int frame_format, int frame_width,
		int frame_height, double frame_aspect, void *data0, void *data1, void *data2 );
	static void raw_overlay_cb(void *user_data, int num_ovl, raw_overlay_t *overlay_array);

public slots:

	void playMrl( QString mrl );
	void play();
	void stop();

	void repaintRequest();
	void startRepaint();
	void stopRepaint();

	void thumb( const QStringList &sl );
	void toggleDeinterlace();

signals:

	void newFrame( const int &frame_format, const int &frame_width,
		const int &frame_height, const double &frame_aspect, void *data0, void *data1, void *data2 );
	void newOverlays( const int &num_ovl, raw_overlay_t *overlay_array );

protected:

	void initializeGL();
	void resizeGL( int, int );
	void paintGL();
	void mouseMoveEvent( QMouseEvent* mev );
	void mousePressEvent( QMouseEvent* mev );
	void wheelEvent( QWheelEvent *e );
	void mouseReleaseEvent( QMouseEvent *mev );
	void mouseDoubleClickEvent( QMouseEvent *e );

private:

	GLVideo *glv;
	GLFunc *func;
	XineEngine *xine;
	XineThumb *xineThumb;
	QString xineMrl;
	Osd *osd;
	QTimer redrawTimer, fpsTimer;
	int numFrames, fps;
	int wantRepaint;
	int animRepaint;

private slots:
	void setFPS();
	void checkNeededRepaint();
};

#endif // VIDEOWIDGET_H
